﻿using System;
using System.Windows.Forms;

namespace NTERA
{
	static class Program
	{
		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main(string[] args)
		{
			var options = new Options();
			options.EraFolder = Environment.GetEnvironmentVariable("ERA");
			if (options.EraFolder == null) {
				options.EraFolder = Environment.CurrentDirectory;
			}

			for (var i = 0; i < args.Length; i++) {
				var arg = args[i];
				if (arg == "--ignoreerrors") {
					options.IgnoreErrors = true;
				}
				else if (arg == "--fontsize") {
					++i;
					int fontsize;
					if (i >= args.Length || !Int32.TryParse(args[i], out fontsize)) {
						System.Console.Write($"Need to specify fontsize (currently {options.FontSize}), e.g.  --fontsize 12\n");
						return;
					}

					options.FontSize = fontsize;
				}
				else if (arg == "--help" || arg == "-h" || arg == "/?") {
					System.Console.Write($@"
NTERA - An ERA game engine, implemented in .net:
	[path to era game]
	--help            - This help message
	--ignoreerrors    - Do not stop on missing function errors etc. (Will likely break the game!)
	--fontsize <size> - Set the font size (Currently {options.FontSize})
					");
					return;
				}
				else if (!arg.StartsWith("-")) {
					options.EraFolder = arg;
				}
				else {
					System.Console.Write($"Error: Argument '{arg}' not understood.  See --help\n");
					return;
				}
			}

			Application.EnableVisualStyles();
			Application.SetCompatibleTextRenderingDefault(false);
			Application.Run(new formMain(options));
		}
	}
}
